package com.ann.exceldemo.web.common;

import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author yangdw
 */
@Slf4j
public class DateUtil {

    private static Logger logger = LoggerFactory.getLogger(DateUtil.class);

    public static final String DATE_FORMAT_PATTERN = "yyyy-MM-dd";
    public static final String TIME_FORMAT_PATTERN = "HH:mm:ss";
    public static final String DATETIME_FORMAT_PATTERN = "yyyy-MM-dd HH:mm:ss";
    public static final String DATETIME_FORMAT_PATTERN_MILL = "yyyy-MM-dd HH:mm:ss.SSS";

    public static String getFormatDate(Date date) {
        if (date == null) {
            return "";
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd");
        return dateFormat.format(date);
    }

    public static String getDateNum(Date date) {
        if (date == null) {
            return "";
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        return dateFormat.format(date);
    }

    public static Integer getYear(Date date) {
        if (date == null) {
            return 0;
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy");
        return StringUtil.toInt(dateFormat.format(date));
    }

    public static Integer getMonth(Date date) {
        if (date == null) {
            return 0;
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM");
        return StringUtil.toInt(dateFormat.format(date));
    }

    public static Integer getDay(Date date) {
        if (date == null) {
            return 0;
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("dd");
        return StringUtil.toInt(dateFormat.format(date));
    }

    /**
     * 时间相差几天几时几分
     */
    public static String getDateStr(Long difftime) {
        String diffTimeStr = "";
        long day = difftime / 86400;
        int hours = (int) (difftime % 86400) / 3600;
        int minutes = (int) ((difftime % 86400) % 3600) / 60;
        int seconds = (int) ((difftime % 86400) % 3600) % 60;
        if (day > 0) {
            diffTimeStr = day + "天" + (hours > 0 ? hours + "小时" : "") + (minutes > 0 ? minutes + "分" : "") + (seconds > 0 ? seconds + "秒" : "");
        } else if (hours > 0) {
            diffTimeStr = hours + "小时" + (minutes > 0 ? minutes + "分" : "") + (seconds > 0 ? seconds + "秒" : "");
        } else if (minutes > 0) {
            diffTimeStr = minutes + "分" + (seconds > 0 ? seconds + "秒" : "");
        } else {
            diffTimeStr = seconds + "秒";
        }
        return diffTimeStr;

    }

    /**
     * @param tmp
     * @param n
     * @return
     */
    @SuppressWarnings("deprecation")
    public static Date addDays(Date tmp, int n) {
        try {
            tmp.setDate(tmp.getDate() + n);
        } catch (Exception e) {
            return null;
        }
        return tmp;
    }

    public static String getFormatDateMonthDay(Date date) {
        if (date == null) {
            return "";
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM-dd");
        return dateFormat.format(date);
    }

    public static String getDate(Date date) {
        if (date == null) {
            return "";
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return dateFormat.format(date);
    }

    public static String getCurrentDate() {
        Date d = new Date();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHH");
        dateFormat.format(d);
        return dateFormat.format(d);
    }

    public static Long getCurrentDateForLong() {
        return new Long(getCurrentDateForInt());
    }

    /**
     * @return
     */
    public static Timestamp getCurrentForTimestamp() {
        return dateToTimestamp(new Date());
    }

    /**
     * @return
     */
    public static Timestamp toTimestamp(String date) {
        return dateToTimestamp(strToDate(date));
    }

    /**
     * 返回当前时间,单位s
     *
     * @return
     */
    public static Long getNowTimeMySql() {
        Date d = new Date();
        return d.getTime() / 1000;
    }


    public static int getCurrentDateForInt() {
        return (int) (new Date().getTime() / 1000);
    }

    public static String formatForStr(Date date, String format) {
        SimpleDateFormat sdf = new SimpleDateFormat(format);
        return sdf.format(date);
    }

    public static Date formatForDate(Date date, String format) {
        return strToDate(formatForStr(date, format), format);
    }

    /**
     * date 日期加上，或减去几天
     *
     * @param date
     * @param dateNum
     * @return
     */
    public static Date getDateInDayAgo(Date date, int dateNum) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(Calendar.DATE, dateNum);
        return cal.getTime();
    }

    /**
     * @param t1
     * @param t2
     * @return 返回秒
     * @throws ParseException
     */
    public static long timeCompare(String t1, String t2) throws ParseException {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        if (t2 == null || "".equals(t2) || t1 == null || "".equals(t1)) {
            return -1;
        }
        Date now = df.parse(t1);
        Date date = df.parse(t2);
        long l = now.getTime() - date.getTime();
        return l;
    }

    /**
     * @param t1
     * @param t2
     * @return 返回秒
     * @throws ParseException
     */
    public static long timeCompare(Date t1, Date t2) {

        if (t2 == null || t1 == null) {
            return -1;
        }
        return t1.getTime() - t2.getTime();
    }

    /**
     * @param senddate
     * @param n
     * @return
     */
    @SuppressWarnings("deprecation")
    public static Date addSecend(Date senddate, int n) {
        Date tmp = (Date) senddate.clone();
        try {
            tmp.setSeconds(tmp.getSeconds() + n);
        } catch (Exception e) {
            return null;
        }

        return tmp;

    }

    /**
     * @param senddate
     * @param n
     * @return
     */
    @SuppressWarnings("deprecation")
    public static Date addMinutes(Date senddate, int n) {

        Date tmp = (Date) senddate.clone();
        try {
            tmp.setMinutes(senddate.getMinutes() + n);
        } catch (Exception e) {
            return null;
        }
        return tmp;

    }

    /**
     * @param @param  sDt
     * @param @return 设定文件
     * @return Long 返回类型
     * @throws
     * @Title: sDt2Long
     */
    public static Long sDt2Long(String sDt) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        Date dt;
        try {
            dt = sdf.parse(sDt);
        } catch (Exception e) {
            return null;
        }
        return dt.getTime();
    }

    /**
     * 根据dateFormat返回date格式化之后的数据形式
     *
     * @param date
     * @param dateFormat
     * @return
     */
    public static String getDateStr(Date date, String dateFormat) {
        if (date == null || StringUtils.isEmpty(dateFormat)) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        return sdf.format(date);
    }


    /**
     * unix时间戳转化为Java时间
     */
    public static String timeStamp2Date(long timestamp) {
        String date = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date(timestamp * 1000));
        return date;
    }

    /**
     * unix时间戳转化为Java时间
     */
    public static Date LongToDate(long timestamp) {
        return new Date(timestamp * 1000);
    }

    /**
     * unix时间戳转化为Java时间
     */
    public static String LongToDateStr(long timestamp) {
        return DateUtil.getDate(new Date(timestamp * 1000));
    }

    public static Long getLongByDate(Date d) {
        if (d == null) {
            return null;
        }
        return d.getTime() / 1000;
    }

    public static Long getLongByDate(Object d) {
        if (d == null) {
            return 0l;
        }

        Date t = strToDate(d.toString());
        if (t == null) {
            return null;
        }
        return t.getTime() / 1000;
    }

    public static Long getMysqlTime() {
        return getLongByDate(new Date());
    }

    public static String getSubDateStr(Date first, Date last) {

        long l = first.getTime() - last.getTime();
        long day = l / (24 * 60 * 60 * 1000);
        long hour = (l / (60 * 60 * 1000) - day * 24);
        long min = ((l / (60 * 1000)) - day * 24 * 60 - hour * 60);
        long s = (l / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
        if (l < 1) {
            return null;
        } else {

            return "" + day + "天" + hour + "小时" + min + "分" + s + "秒";
        }

    }

    /**
     * 方法作用：计算两个时间戳的差 输入参数：endTime截至时间戳，startTime开始时间戳 返回值：字符串
     *
     * @return
     */
    public static String getDiffTime(long endTime, long startTime) {
        String diffTime = "0";
        long difftime = endTime - startTime;

        long day = difftime / 86400;
        int hours = (int) (difftime % 86400) / 3600;
        int minutes = (int) ((difftime % 86400) % 3600) / 60;
        int seconds = (int) ((difftime % 86400) % 3600) % 60;
        if (day > 0) {
            diffTime = day + "天" + (hours > 0 ? hours + "小时" : "") + (minutes > 0 ? minutes + "分" : "") + (seconds > 0 ? seconds + "秒" : "");
        } else if (hours > 0) {
            diffTime = hours + "小时" + (minutes > 0 ? minutes + "分" : "") + (seconds > 0 ? seconds + "秒" : "");
        } else if (minutes > 0) {
            diffTime = minutes + "分" + (seconds > 0 ? seconds + "秒" : "");
        } else {
            diffTime = seconds + "秒";
        }
        return diffTime;
    }

    /**
     * 作用：格式化显示时间
     */
    public static String showTime(long time) {
        String strTime = "0";

        long day = time / 86400;
        int hours = (int) (time % 86400) / 3600;
        int minutes = (int) ((time % 86400) % 3600) / 60;
        int seconds = (int) ((time % 86400) % 3600) % 60;
        if (day > 0) {
            strTime = day + "天" + (hours > 0 ? hours + "小时" : "") + (minutes > 0 ? minutes + "分" : "") + (seconds > 0 ? seconds + "秒" : "");
        } else if (hours > 0) {
            strTime = hours + "小时" + (minutes > 0 ? minutes + "分" : "") + (seconds > 0 ? seconds + "秒" : "");
        } else if (minutes > 0) {
            strTime = minutes + "分钟" + (seconds > 0 ? seconds + "秒" : "");
        } else {
            strTime = seconds + "秒";
        }
        return strTime;
    }

    /**
     * 字符串转换到时间格式
     *
     * @param dateStr   需要转换的字符串
     * @param formatStr 需要格式的目标字符串 举例 yyyy-MM-dd
     * @return Date 返回转换后的时间
     * @throws ParseException 转换异常
     */
    public static Date strToDate(String dateStr, String formatStr) {
        DateFormat sdf = new SimpleDateFormat(formatStr);
        Date date = null;
        try {
            date = sdf.parse(dateStr);
        } catch (ParseException e) {
            logger.error("", e);
        }
        return date;
    }

    /**
     * unix时间戳转化为Java时间
     */
    public static Date strToDate(String dateStr) {
        DateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = null;
        try {
            date = sdf.parse(dateStr);
        } catch (ParseException e) {
            logger.error("", e);
        }
        return date;

    }

    /**
     * 获得date的一天的开始,如2013-11-25 00:00:00
     *
     * @param date
     * @return
     */
    public static String getStartDateStr(Date date) {
        String ret = null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            dateFormat.setLenient(false);
            ret = dateFormat.format(date);
            ret += " 00:00:00";
        } catch (Exception e) {
        }
        return ret;
    }

    /**
     * 获得date的一天的开始,如2013-11-25 00:00:00
     *
     * @param date
     * @return
     */
    public static Date getStartDate(Date date) {
        String ret = getStartDateStr(date);
        return strToDate(ret);
    }

    public static Date getEndDate(Date date) {
        String ret = getEndDateStr(date);
        return strToDate(ret);

    }

    public static String getEndDateStr(Date date) {
        String ret = null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            dateFormat.setLenient(false);
            date = addDates(date, 1);
            ret = dateFormat.format(date);
            ret += " 00:00:00";
        } catch (Exception e) {
        }
        return ret;
    }

    /**
     * @param date
     * @param n
     * @return
     */
    @SuppressWarnings("deprecation")
    public static Date addDates(Date date, int n) {
        Date tmp = new Date();
        try {
            tmp.setDate(date.getDate() + n);
        } catch (Exception e) {
            return null;
        }
        return tmp;
    }

    /**
     * 根据@param num返回之前相连的date,正序
     *
     * @param num
     * @return
     */
    public static Date[] getBefRecentDates(int num) {
        if (num <= 0) {
            return null;
        }
        Date[] dates = new Date[num];
        Date now = new Date();
        for (int i = num - 1; i >= 0; i--) {
            dates[i] = now;
            now = getDateInDayAgo(now, -1);
        }
        return dates;
    }

    /**
     * java.util.Date -> Timestamp
     *
     * @param d
     * @return
     */
    public static Timestamp dateToTimestamp(Date d) {
        if (d == null)
            return null;
        Timestamp s = new Timestamp(d.getTime());
        return s;
    }

    /**
     * Timestamp -> java.util.Date
     *
     * @param t
     * @return
     */
    public static Date timestampToDate(Timestamp t) {
        if (t == null)
            return null;
        Date d = new Date(t.getTime());
        return d;
    }

    /**
     * 获得前一天的日期如:2013-11-25 00:00:00
     *
     * @return
     */
    public static String getLastDateStr() {
        Date d = new Date();
        int before = -1;
        // date before
        Date db = DateUtil.getDateInDayAgo(d, before);

        // 获得前一天的整点日期 2013-11-25 00:00:00
        String dbStr = DateUtil.getStartDateStr(db);
        return dbStr;
    }

    /**
     * 获得前一天的日期 2013-11-25 00:00:00
     *
     * @return
     */
    public static Date getLastDate() {
        Date d = new Date();
        int before = -1;
        // date before
        Date db = DateUtil.getDateInDayAgo(d, before);
        // 获得前一天的整点日期 2013-11-25 00:00:00
        String dbStr = DateUtil.getStartDateStr(db);
        return strToDate(dbStr, "yyyy-MM-dd HH:mm:ss");
    }


    /**
     * 单位 s
     *
     * @return
     */
    public static Long getNowSecond() {
        return new Date().getTime() / 1000;
    }

    /**
     * 得到当前时间的前一天
     *
     * @return
     */
    public static Date getYesterday() {
        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        date = calendar.getTime();
        return date;
    }

    /**
     * 时间戳转字符串日期
     *
     * @param timestamp
     * @param format
     * @return
     */
    public static String timeToString(long timestamp, String format) {

        if (timestamp <= 0) {
            return "";
        }
        if (StringUtil.isEmpty(format)) {
            format = DATETIME_FORMAT_PATTERN;
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat(format);
        return dateFormat.format(new Date(timestamp));
    }

    public static long minBeforeNow(Date date) {
        long nowTime = Calendar.getInstance().getTimeInMillis();
        Calendar comCal = Calendar.getInstance();
        comCal.setTime(date);
        long comTime = comCal.getTimeInMillis();
        long i = nowTime - comTime;//得到的毫秒书
        return i / (1000 * 60);//分数
    }

    /**
     * 增加日期
     */
    public static Date addDate(Date source, Integer dateCount) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(source);
        calendar.add(Calendar.DAY_OF_MONTH, dateCount);
        return calendar.getTime();
    }

    /**
     * 获取整形 int 日期  单位 秒
     *
     * @param date 日期
     */
    public static int getDateInSecond(Date date) {
        return (int) (date.getTime() / 1000);
    }

    /**
     * 将startTime-endTime分割为指定size的时间段(无法均分的情况下加到最后一个时间段)
     *
     * @param startTime 起始时间(秒级时间戳)
     * @param endTime   结束时间(秒级时间戳)
     * @param size      时间段个数
     * @return 时间段列表(大小为size)
     */
    public static List<Pair<Long, Long>> splitTimeRange(long startTime, long endTime, int size) {
        if (startTime >= endTime) {
            throw new RuntimeException("startTime必须小于endTime");
        }
        if (size <= 0) {
            throw new RuntimeException("size必须大于0");
        }
        if (size == 1) {
            return Lists.newArrayList(Pair.of(startTime, endTime));
        }
        long range = endTime - startTime;
        if (size > range) {
            throw new RuntimeException("size必须小于等于时间跨度");
        }
        long step = range / size;
        List<Pair<Long, Long>> resultList = Lists.newArrayList();
        for (int index = 0; index < size; index++) {
            Long left = startTime + index * step;
            Long right = left + step;
            if (index == size - 1) {
                right = endTime;
            }
            Pair<Long, Long> pair = Pair.of(left, right);
            resultList.add(pair);
        }
        return resultList;
    }

    /**
     * 通过日期解析当前日期起始和结束unix时间戳
     *
     * @param dateStr 日期字符串(格式:yyyy-MM-dd)
     * @return 当前日期起始和结束unix时间戳
     */
    public static Pair<Long, Long> parseStartEndTimeByDate(String dateStr) {
        if (org.apache.commons.lang3.StringUtils.isBlank(dateStr)) {
            throw new RuntimeException("date不能为空");
        }
        try {
            Date startDate = org.apache.commons.lang3.time.DateUtils.parseDate(dateStr, "yyyy-MM-dd");
            Long startTime = startDate.getTime() / 1000;
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(startDate);
            calendar.add(Calendar.DATE, 1);
            Date endDate = calendar.getTime();
            Long endTime = endDate.getTime() / 1000;
            return Pair.of(startTime, endTime);
        } catch (Exception e) {
            log.error("日期转化异常, dataStr:" + dateStr, e);
            throw new RuntimeException("日期转化异常, dataStr:" + dateStr, e);
        }
    }

    /**
     * 通过[startDate, endDate)解析出包含的日期列表
     *
     * @param startDate 起始日期
     * @param endDate   结束日期
     * @return 包含的日期列表
     */
    public static List<String> parseDateList(Date startDate, Date endDate) {
        if (null == startDate || null == endDate) {
            return Lists.newArrayList();
        }
        List<String> dateList = Lists.newArrayList();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        while (calendar.getTime().before(endDate)) {
            dateList.add(DateFormatUtils.format(calendar.getTime(), "yyyy-MM-dd"));
            calendar.add(Calendar.DATE, 1);
        }
        return dateList;

    }
}