package com.ann.exceldemo.web.common;

import com.ann.exceldemo.web.constant.KEY_BIZ;
import net.sf.json.JSONObject;
import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * @author 汤学旭
 * @date 2013-5-7 下午2:50:55
 */
public final class HttpUtil {
    private static final String CHAR_SET = "UTF-8";
    private static final String CONTENT_TYPE = "application/json;charset=utf-8";
    private static final String SOCKET_TIMEOUT = "http.socket.timeout";
    private static final int TIME_OUT = 1000 * 59;
    private static Logger logger = LoggerFactory.getLogger(HttpUtil.class);

    /**
     * @param @param  url
     * @param @param  params
     * @param @return
     * @param @throws Exception 设定文件
     * @return String 返回类型
     * @throws
     * @Title: doPost
     */
    public static void doPost(String url, String params) {
        HttpClient httpClient  = new HttpClient();
        GetMethod getMethod = new GetMethod(url);
        try {
            url = url + "?data=" + URLEncoder.encode(params, "UTF-8");
            getMethod.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler());
            // 执行getMethod
            int statusCode = httpClient.executeMethod(getMethod);
            if (statusCode != HttpStatus.SC_OK) {
                throw new RuntimeException(url + params + " do post error:" + statusCode);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            httpClient.getHttpConnectionManager().closeIdleConnections(0L);
            getMethod.releaseConnection();
        }
    }

    /**
     * 获得eth0网络接口ip地址
     *
     * @return
     */
    public static String getEth0Ip() {
        return getIpByInterfaceName("eth0");
    }

    /**
     * 获得eth1网络接口ip地址
     *
     * @return
     */
    public static String getEth1Ip() {
        return getIpByInterfaceName("eth1");
    }

    /**
     * 根据 interface name 获得机器ip地址
     *
     * @param interfaceName
     * @return
     */
    private static String getIpByInterfaceName(String interfaceName) {
        if (interfaceName == null || interfaceName.equals("")) {
            return null;
        }

        Enumeration<NetworkInterface> allNetInterfaces = null;
        try {
            allNetInterfaces = NetworkInterface.getNetworkInterfaces();
        } catch (java.net.SocketException e) {
            logger.error("", e);
        }
        if (allNetInterfaces == null) {
            return null;
        }
        InetAddress ip = null;
        while (allNetInterfaces.hasMoreElements()) {
            NetworkInterface netInterface = allNetInterfaces.nextElement();
            String netInterfaceName = netInterface.getName();
            if (!interfaceName.equals(netInterfaceName))
                break;
            Enumeration<InetAddress> addresses = netInterface.getInetAddresses();
            while (addresses.hasMoreElements()) {
                ip = addresses.nextElement();
                if (ip != null && ip instanceof Inet4Address) {
                    return ip.getHostAddress();
                }
            }
        }
        return null;
    }

    /**
     * @param response
     * @param message  //
     */
    public static void writeResponse(HttpServletResponse response, String message) {
//        response.setHeader("Content-type", "text/html;charset=UTF-8");
        response.setContentType(CONTENT_TYPE);
        response.setCharacterEncoding("utf-8");
        try (PrintWriter writer = response.getWriter()) {
            writer.write(message);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static void writeX5Msg(HttpServletResponse response, KEY_BIZ keybiz, JSONObject body) {
        Map<String, Object> map = new HashMap<String, Object>();
        Map<String, String> header = new HashMap<String, String>();
        header.put("code", keybiz.getValue());
        header.put("desc", keybiz.getDesc());
        map.put("header", header);
        if (body != null) {
            map.put("body", body);
        } else {
            map.put("body", header);
        }
        String ret = JSONObject.fromObject(map).toString();
        writeResponse(response, ret);
    }

    /**
     * @param response
     * @param keybiz
     */
    public static void writeX5Msg(HttpServletResponse response, KEY_BIZ keybiz) {
        JSONObject map = new JSONObject();
        JSONObject header = new JSONObject();
        header.put("code", keybiz.getValue());
        header.put("desc", keybiz.getDesc());
        map.put("header", header);
        JSONObject body = new JSONObject();
        body.put("code", keybiz.getValue());
        body.put("desc", keybiz.getDesc());
        map.put("body", body);
        String ret = JSONObject.fromObject(map).toString();
        writeResponse(response, ret);
    }

    /**
     * @param response
     */
    public static void writeX5Msg(HttpServletResponse response, MyException mie) {
        Map<String, Map<String, String>> map = new HashMap<String, Map<String, String>>();
        Map<String, String> body = new HashMap<String, String>();
        body.put("code", mie.getMsgCode());
        body.put("desc", mie.getMsg());
        map.put("body", body);
        map.put("header", body);
        String ret = JSONObject.fromObject(map).toString();
        writeResponse(response, ret);
    }

    public static void writeX5Msg(HttpServletResponse response, Exception exception) {
        String code = "";
        String desc = "";
        if (exception instanceof MyException) {
            MyException xiaomiException = (MyException) exception;
            code = xiaomiException.getKEYBIZ().value;
            desc = xiaomiException.getKEYBIZ().getDesc();
        } else {
            code = KEY_BIZ.UN_KNOW.value;
            desc = exception.getMessage();
        }

        // header
        Map<String, String> header = new HashMap<String, String>();
        header.put("code", code);
        header.put("desc", desc);
        JSONObject resultJSON = new JSONObject();
        resultJSON.put("header", header);
        writeResponse(response, resultJSON.toString());
    }

    public static void writeMiExceptionMsg(HttpServletResponse response, KEY_BIZ keyBiz, JSONObject body) {
        String ret = JSONObject.fromObject(body).toString();
        writeResponse(response, ret);
    }

    /**
     * @return
     */
    public static HttpServletRequest getRequest() {
        if (RequestContextHolder.getRequestAttributes() == null || (RequestContextHolder.getRequestAttributes()) == null)
            return null;
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        return request;
    }


    public static String doPost(String url, Map<String, String> params, String charset, boolean pretty) {
        StringBuffer response = new StringBuffer();
        HttpClient client = new HttpClient();
        HttpMethod method = new PostMethod(url);
        //设置Http Post数据
        if (params != null) {
            HttpMethodParams p = new HttpMethodParams();
            for (Map.Entry<String, String> entry : params.entrySet()) {
                p.setParameter(entry.getKey(), entry.getValue());
            }
            method.setParams(p);
        }
        try {
            client.executeMethod(method);
            if (method.getStatusCode() == HttpStatus.SC_OK) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(method.getResponseBodyAsStream(), charset));
                String line;
                while ((line = reader.readLine()) != null) {
                    if (pretty)
                        response.append(line).append(System.getProperty("line.separator"));
                    else
                        response.append(line);
                }
                reader.close();
            }
        } catch (IOException e) {
            logger.error("", e);
        } finally {
            method.releaseConnection();
        }
        return response.toString();
    }

}
