package com.ann.exceldemo.web.controller;

import com.ann.exceldemo.web.common.DateUtil;
import com.ann.exceldemo.web.common.FileUtil;
import com.ann.exceldemo.web.common.HttpUtil;
import com.ann.exceldemo.web.common.StringUtil;
import com.ann.exceldemo.web.constant.AccessControlEnum;
import com.ann.exceldemo.web.constant.KEY_BIZ;
import com.ann.exceldemo.web.constant.TaskTypeEnum;
import com.ann.exceldemo.web.dto.*;
import com.ann.exceldemo.web.json.JSONUtil;
import com.ann.exceldemo.web.service.OrderService;
import com.ann.exceldemo.web.service.TaskConfigService;
import com.ann.exceldemo.web.service.WaybillService;
import com.google.common.collect.Lists;
import com.grapecity.documents.excel.IWorksheet;
import com.grapecity.documents.excel.Workbook;
import net.sf.json.JSONObject;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.util.*;

@CrossOrigin
@RestController
public class ExcelController {
    @Resource
    private TaskConfigService taskConfigService;
    @Resource
    private OrderService orderService;
    @Resource
    private WaybillService waybillService;

    /**
     * 查询任务配置列表
     *
     * @param response
     */
    @RequestMapping(value = "/query/task/config", produces = {"application/json;charset=utf-8"})
    public void queryTaskConfig(HttpServletResponse response) {
        try {
            JSONObject result = new JSONObject();
            List<TaskConfig> taskConfigs = taskConfigService.queryAll();
            List<TaskConfigDto> taskConfigDtoList = buildResult(taskConfigs);
            result.put("items", taskConfigDtoList);

            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS, result);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    /**
     * 更新任务配置
     *
     * @param response
     */
    @RequestMapping(value = "/update/task/status", produces = {"application/json;charset=utf-8"})
    public void updateTaskStatus(HttpServletResponse response, HttpServletRequest request) {
        try {
            Map<String, String[]> parameterMap = request.getParameterMap();
            Integer id = Integer.valueOf(parameterMap.get("id")[0]);
            Integer status = Integer.valueOf(parameterMap.get("taskStatus")[0]);
            String operator = parameterMap.get("operator")[0];
            taskConfigService.updateById(id, status, operator);
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    /**
     * 更新任务配置
     *
     * @param response
     */
    @RequestMapping(value = "/update/save/type", produces = {"application/json;charset=utf-8"})
    public void updateSaveType(HttpServletResponse response, HttpServletRequest request) {
        try {
            Map<String, String[]> parameterMap = request.getParameterMap();
            Integer id = Integer.valueOf(parameterMap.get("id")[0]);
            String type = parameterMap.get("saveType")[0];
            String operator = parameterMap.get("operator")[0];
            taskConfigService.updateSaveType(id, type, operator);
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    /**
     * 构造返回结果
     *
     * @param taskConfigs
     * @return
     */
    private List<TaskConfigDto> buildResult(List<TaskConfig> taskConfigs) throws InvocationTargetException, IllegalAccessException {
        List<TaskConfigDto> configDtoList = Lists.newArrayList();
        for (TaskConfig taskConfig : taskConfigs) {
            TaskConfigDto taskConfigDto = new TaskConfigDto();
            BeanUtils.copyProperties(taskConfigDto, taskConfig);
            taskConfigDto.setCreateTime(taskConfig.getCreateTime().getTime() / 1000);
            taskConfigDto.setUpdateTime(taskConfig.getUpdateTime().getTime() / 1000);
            taskConfigDto.setFilePath(getFileName(getBasePath(taskConfig.getTaskType())));
            configDtoList.add(taskConfigDto);
        }
        return configDtoList;
    }

    /**
     * 查询订单列表
     *
     * @param response
     */
    @RequestMapping(value = "/query/order/list", produces = {"application/json;charset=utf-8"})
    public void queryOrderList(HttpServletResponse response) {
        try {
            JSONObject result = new JSONObject();
            List<Order> orders = orderService.queryAll();
            List<OrderDto> orderDtoList = buildOrderResult(orders);
            //仅返回两条数据用于模版编辑
            result.put("items", orderDtoList.subList(0, 2));
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS, result);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    /**
     * 查询订单列表
     *
     * @param response
     */
    @RequestMapping(value = "/query/waybill/list", produces = {"application/json;charset=utf-8"})
    public void queryWaybillList(HttpServletResponse response) {
        try {
            JSONObject result = new JSONObject();
            List<Waybill> waybills = waybillService.queryAll();
            List<WaybillDto> waybillDtoList = buildWaybillResult(waybills);
            result.put("items", waybillDtoList);
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS, result);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    /**
     * 更新任务配置
     *
     * @param response
     */
    @CrossOrigin
    @RequestMapping(value = "/update/task/template", produces = {"application/json;charset=utf-8"})
    public void updateTaskTemplate(HttpServletResponse response, HttpServletRequest request, @RequestBody UpdateTemplateVo templateVo) {
        try {
            taskConfigService.updateContext(templateVo.getId(), templateVo.getContext(), templateVo.getOperator());
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    /**
     * 更新任务配置
     *
     * @param response
     */
    @CrossOrigin
    @RequestMapping(value = "/insert/order", produces = {"application/json;charset=utf-8"})
    public void insertOrder(HttpServletResponse response, @RequestBody Order order) {
        try {
            order.setCreateTime(new Date());
            order.setUpdateTime(new Date());
            orderService.insert(order);
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }


    /**
     * 更新任务配置
     *
     * @param response
     */
    @RequestMapping(value = "/download/file", produces = {"application/json;charset=utf-8"})
    public void downLoadFile(HttpServletResponse response, HttpServletRequest request) {
        try {
            Map<String, String[]> parameterMap = request.getParameterMap();
            String taskType = parameterMap.get("taskType")[0];
            String basePath = getBasePath(taskType);
            // 文件本地位置
            String fileName = getFileName(basePath);
            File file = new File(basePath + fileName);
            FileUtil.downloadFile(file, request, response, fileName);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    /**
     * 接受前端返回的文件信息
     *
     * @param response
     */
    @RequestMapping(value = "/send/file/blob", produces = {"application/json;charset=utf-8"})
    public void sendFileBlob(HttpServletResponse response, HttpServletRequest request) {
        MultipartHttpServletRequest multiRequest = (MultipartHttpServletRequest) request;
        Iterator<String> iter = multiRequest.getFileNames();
        while (iter.hasNext()) {
            MultipartFile multipartFile = multiRequest.getFile(iter.next());
            if (null != multipartFile) {
                String fileName = multipartFile.getOriginalFilename();
                if (fileName == null || fileName.trim().equals("")) {
                    continue;
                }
                //todo 具体的业务逻辑
            }
        }
    }

    @GetMapping(value = "/upload/file")
    public void download(String filePath, HttpServletResponse response) throws Exception {
        ServletOutputStream out = null;
        ByteArrayOutputStream byteArrayOutputStream = null;
        try {
            FileInputStream inputStream = new FileInputStream(filePath);
            String[] filePaths = StringUtil.split(filePath, "/");
            byte[] buffer = new byte[1024];
            int len;
            byteArrayOutputStream = new ByteArrayOutputStream();
            while ((len = inputStream.read(buffer)) != -1) {
                byteArrayOutputStream.write(buffer, 0, len);
            }
            response.addHeader("Content-Disposition", "attachment;filename=" + filePaths[filePaths.length - 1]);
            response.addHeader("Content-Length", "" + byteArrayOutputStream.size());
            response.setHeader("filename", filePaths[filePaths.length - 1]);
            response.setContentType("application/octet-stream");
            out = response.getOutputStream();
            out.write(byteArrayOutputStream.toByteArray());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            // 关闭文件流
            if (null != byteArrayOutputStream) {
                byteArrayOutputStream.flush();
                byteArrayOutputStream.close();
            }
            if (null != out) {
                out.flush();
                out.close();
            }
        }
    }


    private List<WaybillDto> buildWaybillResult(List<Waybill> waybills) throws InvocationTargetException, IllegalAccessException {
        List<WaybillDto> waybillDtos = Lists.newArrayList();
        for (Waybill waybill : waybills) {
            WaybillDto waybillDto = new WaybillDto();
            BeanUtils.copyProperties(waybillDto, waybill);
            waybillDto.setCreateTime(DateUtil.getDate(waybill.getCreateTime()));
            waybillDto.setUpdateTime(DateUtil.getDate(waybill.getUpdateTime()));
            waybillDto.setOutTime(DateUtil.getDate(waybill.getOutTime()));
            waybillDtos.add(waybillDto);
        }
        return waybillDtos;
    }

    /**
     * 构建返回值
     *
     * @param orders
     * @return
     */
    private List<OrderDto> buildOrderResult(List<Order> orders) throws InvocationTargetException, IllegalAccessException {
        List<OrderDto> orderDtoList = Lists.newArrayList();
        for (Order order : orders) {
            OrderDto orderDto = new OrderDto();
            BeanUtils.copyProperties(orderDto, order);
            orderDto.setCreateTime(DateUtil.getDate(order.getCreateTime()));
            orderDto.setUpdateTime(DateUtil.getDate(order.getUpdateTime()));
            orderDtoList.add(orderDto);
        }
        return orderDtoList;
    }

    /**
     * 获取指定文件夹下的文件路径下文件名称
     */
    public static String getFileName(String basePath) {
        //获取路径
        File f = new File(basePath);
        if (!f.exists()) {
            return "";
        }
        String suffix = "";
        //用数组接收
        File[] fa = f.listFiles();
        long maxFile = 0L;
        for (int i = 0; i < fa.length; i++) {
            String[] paths = StringUtil.split(fa[i].getName(), ".");
            suffix = paths[1];
            maxFile = Math.max(maxFile, Long.parseLong(paths[0]));
        }
        return maxFile + "." + suffix;
    }

    /**
     * 根据不同的类型获取文件base路径
     *
     * @param taskType
     * @return
     */
    private static String getBasePath(String taskType) {
        String path = "";
        if (taskType.equals(TaskTypeEnum.order.getName())) {
            path = "//Users/yak/myProject/exceldemo/src/main/resources/static/order/";
        } else if (taskType.equals(TaskTypeEnum.waybill.getName())) {
            path = "//Users/yak/myProject/exceldemo/src/main/resources/static/waybill/";
        }
        return path;
    }

    /**
     * 根据权限屏蔽数据行
     * @param response
     * @param request
     */
    @RequestMapping(value = "/get/user/access", produces = {"application/json;charset=utf-8"})
    public static void getUserAccess(HttpServletResponse response, HttpServletRequest request) {
        try {
            Map<String, String[]> parameterMap = request.getParameterMap();
            String name = parameterMap.get("name")[0];
            Workbook workbook = new Workbook();
            workbook.open("userAccess.xlsx");
            if (!StringUtil.isEmpty(name) && !name.equals(AccessControlEnum.ALL.getName())) {
                String userAccess = AccessControlEnum.getAccessByName(name);
                IWorksheet worksheet = workbook.getWorksheets().get(0);
                int rowStart = worksheet.getUsedRange().getRow();
                int rowEnd = worksheet.getUsedRange().getLastRow();
                for (int i = rowStart; i <= rowEnd; i++) {
                    if (!worksheet.getRange(i, 2).getText().equals(userAccess)) {
                        worksheet.getRange(i, 2).getEntireRow().setHidden(true);
                    }
                }
            }
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS, JSONUtil.encodeToJsonlibObject(workbook.toJson()));
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }

    @RequestMapping(value = "/save/file", produces = {"application/json;charset=utf-8"})
    public static void saveFile(HttpServletResponse response, @RequestBody UpdateTemplateVo templateVo) {
        try {
            Workbook workbook = new Workbook();
            if (!StringUtil.isEmpty(templateVo.getContext())) {
                workbook.fromJson(templateVo.getContext());
                workbook.save("person.xlsx");
            }
            HttpUtil.writeX5Msg(response, KEY_BIZ.SUCCESS);
        } catch (Exception e) {
            HttpUtil.writeX5Msg(response, e);
        }
    }
}
