package com.ann.exceldemo.web.job;

import com.ann.exceldemo.web.common.DateUtil;
import com.ann.exceldemo.web.common.SpringUtil;
import com.ann.exceldemo.web.common.WeChatBotUtils;
import com.ann.exceldemo.web.constant.TaskTypeEnum;
import com.ann.exceldemo.web.dto.Order;
import com.ann.exceldemo.web.dto.*;
import com.ann.exceldemo.web.json.JSONUtil;
import com.ann.exceldemo.web.service.OrderService;
import com.ann.exceldemo.web.service.WaybillService;
import com.google.common.collect.Lists;
import com.grapecity.documents.excel.*;
import com.grapecity.documents.excel.drawing.IShape;
import com.grapecity.documents.excel.drawing.ImageType;
import com.grapecity.documents.excel.drawing.RowCol;
import net.sf.json.JSONObject;
import org.apache.commons.beanutils.BeanUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.springframework.stereotype.Component;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 实现报表数据的查询和打印
 */
@Component
public class ExcelJob implements Job {
    private static final String BOT_URL = "https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=980f1397-376b-4ef9-a2dc-beb03bb401e7";
    private static final String ORDER_BASE_PATH = "/Users/yak/myProject/exceldemo/src/main/resources/static/order/";
    private static final String WAYBILL_BASE_PATH = "/Users/yak/myProject/exceldemo/src/main/resources/static/waybill/";


    @Override
    public void execute(JobExecutionContext jobExecutionContext) {
        WeChatBotUtils weChatBotUtils = new WeChatBotUtils(BOT_URL, false);
        TaskConfig taskConfig = (TaskConfig) jobExecutionContext.getJobDetail().getJobDataMap().get("taskConfig");
        JSONObject temp = JSONUtil.encodeToJsonlibObject(taskConfig.getTaskContext());
        JSONObject sheets = (JSONObject) temp.get("sheets");
        if (jobExecutionContext.getJobDetail().getKey().getName().equals(TaskTypeEnum.order.getDescribe())) {
            JSONObject waybill = (JSONObject) sheets.get("order");
            ColumnsVo columnsVo = JSONUtil.decode(String.valueOf(waybill), ColumnsVo.class);
            OrderService orderService = SpringUtil.getBean(OrderService.class);
            List<Order> orders = orderService.queryAll();
            List<OrderDto> orderDtoList = buildOrderResult(orders);
            //根据模版打开一个工作簿
            Workbook workbook = new Workbook();
            workbook.fromJson(taskConfig.getTaskContext());
            //一个表单存储数据 一个表单展示图表
            IWorksheet worksheet = workbook.getWorksheets().get(0);
            worksheet.setDataSource(orderDtoList);
            worksheet.setAutoGenerateColumns(false);
            IShape iShape = worksheet.getShapes().get(0);
            Object[][] chartData = new Object[orderDtoList.size() + 1][columnsVo.getColumns().size()];
            Object[][] viewData = (Object[][]) worksheet.getUsedRange().getValue();
            Object[] firstCol = new Object[columnsVo.getColumns().size()];
            for (int i = 0; i < columnsVo.getColumns().size(); i++) {
                if (i == 0) {
                    firstCol[0] = null;
                } else {
                    firstCol[i] = columnsVo.getColumns().get(i).getName();
                }
            }
            chartData[0] = firstCol;
            System.arraycopy(viewData, 0, chartData, 1, viewData.length);
            worksheet.getRange(0, 0, orderDtoList.size() + 1, columnsVo.getColumns().size()).setValue(chartData);
//            iShape.getChart().getSeriesCollection().extend(worksheet.getRange(0,0,orderDtoList.size()+1,columnsVo.getColumns().size()), RowCol.Columns, true);
            iShape.getChart().setAutoScaling(true);
            iShape.getChart().setSourceData(worksheet.getRange(0, 0, orderDtoList.size() + 1, columnsVo.getColumns().size()), RowCol.Columns);
            long now = new Date().getTime();
            String filePath = ORDER_BASE_PATH + now + ".pdf";
            String msgPath = ORDER_BASE_PATH + now + ".png";
            FileOutputStream outputStream = null;
            try {
                outputStream = new FileOutputStream(msgPath);
                worksheet.getShapes().get(0).toImage(outputStream, ImageType.PNG);
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } finally {
                try {
                    if (outputStream != null) {
                        outputStream.close();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            //保存为Excel文件
            workbook.save(filePath);
            //将生成的文件的图片发送至微信群
            try {
                weChatBotUtils.sendImgMsg(msgPath);
                weChatBotUtils.sendFileMsg(filePath);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else if (jobExecutionContext.getJobDetail().getKey().getName().equals(TaskTypeEnum.waybill.getDescribe())) {
            JSONObject waybill = (JSONObject) sheets.get("waybill");
            ColumnsVo columnsVo = JSONUtil.decode(String.valueOf(waybill), ColumnsVo.class);
            WaybillService waybillService = SpringUtil.getBean(WaybillService.class);
            List<Waybill> waybills = waybillService.queryAll();
            List<WaybillDto> waybillDtos = buildWaybillResult(waybills);
            WaybillDataSource waybillDataSource = new WaybillDataSource(waybillDtos);
            Workbook workbook = new Workbook();
            IWorksheet worksheet = workbook.getWorksheets().get(0);
            ITable table = worksheet.getTables().add(worksheet.getRange("B2:" + "N" + waybills.size() + 2), true);
            //不允许列自动转换
            table.setAutoGenerateColumns(false);
            worksheet.getRange("B:N").setColumnWidth(15);
            table.setBindingPath("waybills");
            worksheet.setDataSource(waybillDataSource);
            for (int i = 0; i < columnsVo.getColumns().size(); i++) {
                //数据行绑定列名的属性
                table.getColumns().get(i).setDataField(columnsVo.getColumns().get(i).getName());
                //绑定列名
                table.getColumns().get(i).setName(columnsVo.getColumns().get(i).getName());
            }
            if (taskConfig.getSaveType().equals("PDF")) {
                savePdf(workbook, worksheet, WAYBILL_BASE_PATH + new Date().getTime() + ".pdf");
            } else {
                workbook.save(WAYBILL_BASE_PATH + new Date().getTime() + ".xlsx");
            }
        }
    }

    /**
     * 保存文件为pdf格式
     *
     * @param workbook
     * @param worksheet
     * @param path
     */
    private void savePdf(Workbook workbook, IWorksheet worksheet, String path) {
        FileOutputStream outputStream = null;
        try {
            outputStream = new FileOutputStream(path);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        PrintManager printManager = new PrintManager();
        //初始化page信息
        List<PageInfo> pages = printManager.paginate(workbook);
        //生成新的page信息
        List<PageInfo> newPages = new ArrayList<PageInfo>();
        newPages.add(pages.get(0));
        newPages.get(0).getPageContent().setRange(worksheet.getUsedRange());
        printManager.updatePageNumberAndPageSettings(newPages);
        printManager.savePageInfosToPDF(outputStream, newPages);
        try {
            outputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 构建返回值
     *
     * @param orders
     * @return
     */
    private List<OrderDto> buildOrderResult(List<Order> orders) {
        List<OrderDto> orderDtoList = Lists.newArrayList();
        for (Order order : orders) {
            OrderDto orderDto = new OrderDto();
            try {
                BeanUtils.copyProperties(orderDto, order);
            } catch (IllegalAccessException | InvocationTargetException e) {
                e.printStackTrace();
            }
            orderDto.setCreateTime(DateUtil.getDate(order.getCreateTime()));
            orderDto.setUpdateTime(DateUtil.getDate(order.getUpdateTime()));
            orderDto.setOrderAmount(String.valueOf(order.getOrderAmount()));
            orderDto.setOrderDiscount(String.valueOf(order.getOrderDiscount()));
            orderDto.setShippingFee(String.valueOf(order.getShippingFee()));
            orderDtoList.add(orderDto);
        }
        return orderDtoList;
    }

    private List<WaybillDto> buildWaybillResult(List<Waybill> waybills) {
        List<WaybillDto> waybillDtos = Lists.newArrayList();
        for (Waybill waybill : waybills) {
            WaybillDto waybillDto = new WaybillDto();
            try {
                BeanUtils.copyProperties(waybillDto, waybill);
            } catch (IllegalAccessException | InvocationTargetException e) {
                e.printStackTrace();
            }
            waybillDto.setCreateTime(DateUtil.getDate(waybill.getCreateTime()));
            waybillDto.setUpdateTime(DateUtil.getDate(waybill.getUpdateTime()));
            waybillDto.setOutTime(DateUtil.getDate(waybill.getOutTime()));
            waybillDtos.add(waybillDto);
        }
        return waybillDtos;
    }
}
