package com.ann.exceldemo.web.job;

import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.TriggerKey;

public interface SchedulerListener {
    /**
     * 用于部署JobDetail时调用
     * @param trigger
     */
    public void jobScheduled(Trigger trigger);

    /**
     * 用于卸载JobDetail时调用。
     * @param triggerKey
     */
    public void jobUnscheduled(TriggerKey triggerKey);

    /**
     * 当一个Trigger来到了再也不会触发的状态时调用这个方法。除非这个Job已设置成了持久性，否则它就会从Scheduler中移除。
     * @param trigger
     */
    public void triggerFinalized(Trigger trigger);

    /**
     * Scheduler调用这个方法是发生在一个Trigger或Trigger组被暂停时。假如是Trigger组的话，triggerName参数将为null。
     * @param triggerGroup
     */
    public void triggersPaused(String triggerGroup);

    /**
     * Scheduler调用这个方法是发生在一个Trigger或Trigger组从暂停中恢复时。假如是Trigger组的话，triggerName参数将为null。
     * @param triggerGroup
     */
    public void triggersResumed(String triggerGroup);

    /**
     * 当一个或一组JobDetail暂停时调用这个方法
     * @param jobGroup
     */
    public void jobsPaused(String jobGroup);

    /**
     * 当一个或一组Job从暂停上恢复时调用这个方法。假如是一个Job组，jobName将为null。
     * @param jobGroup
     */
    public void jobsResumed(String jobGroup);

    /**
     * 在Scheduler的正常运行期间产生一个严重错误时调用这个方法
     * @param msg
     * @param cause
     */
    public void schedulerError(String msg, SchedulerException cause);

    /**
     * 当Scheduler开启时，调用该方法。
     */
    public void schedulerStarted();

    /**
     * 当Scheduler处于StandBy模式时，调用该方法。
     */
    public void schedulerInStandbyMode();

    /**
     * 当Scheduler停止时，调用该方法。
     */
    public void schedulerShutdown();

    /**
     * 当Scheduler中的数据被清除时，调用该方法
     */
    public void schedulingDataCleared();
}
