package com.ann.exceldemo.web.job;

import org.quartz.JobExecutionContext;
import org.quartz.Trigger;

public interface TriggerListener {
    /**
     * 用于获取触发器的名称
     * @return
     */
    public String getName();

    /**
     * 当与监听器关联的Trigger被触发，Job上的Execute()方法将被执行时，Scheduler就调用该方法
     * @param trigger
     * @param context
     */
    public void triggerFired(Trigger trigger, JobExecutionContext context);

    /**
     * 在Trigger触发后，Job将要执行时由Scheduler调用这个方法。TriggerListener给了一个选择去否决Job的执行。假如这个方法返回true，这个Job将不会为此次Trigger触发而得到执行
     * @param trigger
     * @param context
     * @return
     */
    public boolean vetoJobExecution(Trigger trigger, JobExecutionContext context);

    /**
     * Scheduler调用这个方法是在Trigger错过触发时。你应该关注此方法中持续时间长的逻辑：在出现许多错过触发的Trigger时，长逻辑会导致骨牌效应。你应当保持这个方法尽量的小。
     * @param trigger
     */
    public void triggerMisfired(Trigger trigger);

    /**
     * Trigger被触发并且完成了Job的执行时，Scheduler调用这个方法。
     * @param trigger
     * @param context
     * @param triggerInstructionCode
     */
    public void triggerComplete(Trigger trigger, JobExecutionContext context, Trigger.CompletedExecutionInstruction triggerInstructionCode);
}
