/*
 * Copyright (c) 2014 xiaomi.com. All Rights Reserved.
 */
package com.ann.exceldemo.web.json;

import com.ann.exceldemo.web.common.ClasspathScanner;
import com.ann.exceldemo.web.common.CodeOfEnumJsonDeserializer;
import com.ann.exceldemo.web.common.CodeOfEnumJsonSerializer;
import com.ann.exceldemo.web.common.DateUtil;
import com.ann.exceldemo.web.constant.CodeOfEnum;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.module.SimpleModule;

import org.apache.commons.collections.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.Set;

/**
 * ObjectMapper对象
 *
 * @author: dengliren Date: 2014-09-05 下午7:04
 * @version: \$Id$
 */
public class SimpleObjectMapper extends ObjectMapper {

    public SimpleObjectMapper() {
        super();
        // 当找不到对应的序列化器时 忽略此字段
        configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false);
        setVisibility(PropertyAccessor.FIELD, JsonAutoDetect.Visibility.ANY);
        // 配置为true表示mapper接受只有一个元素的数组的反序列化
        configure(DeserializationFeature.ACCEPT_SINGLE_VALUE_AS_ARRAY, true);
        // 当找不到对应的序列化器时 忽略此字段
        disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        // 新版的jackson设置mapper的方法，功能同上
        getDeserializationConfig().without(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
        // 定义针对日期类型的反序列化时的数据格式
        setDateFormat(new SimpleDateFormat(DateUtil.DATETIME_FORMAT_PATTERN));
        configure(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS, false);

        SimpleModule module = new SimpleModule();
        module.addSerializer(CodeOfEnum.class, new CodeOfEnumJsonSerializer<CodeOfEnum>());

        // 扫描codeof enum 并注册相应的JsonDeserializer
        Set<Class<?>> codeOfEnums = new ClasspathScanner("com.ann.exceldemo.web", new ClasspathScanner.ClassFilter() {
            public boolean accept(Class<?> clazz) {
                return CodeOfEnum.class.isAssignableFrom(clazz);
            }
        }).scan();

        if (CollectionUtils.isNotEmpty(codeOfEnums)) {
            for (Class<?> c : codeOfEnums) {
                JsonDeserializer codeOfEnumJsonDeserializer = new CodeOfEnumJsonDeserializer((Class<? extends CodeOfEnum>) c);
                module.addDeserializer(c, codeOfEnumJsonDeserializer);
            }
        }

        registerModule(module);
    }
}
