package com.ann.exceldemo.web.schedule;

import com.ann.exceldemo.web.dto.TaskConfig;
import com.ann.exceldemo.web.job.ExcelJob;
import com.ann.exceldemo.web.job.MyJobListener;
import com.ann.exceldemo.web.service.TaskConfigService;
import com.google.common.collect.Maps;
import org.quartz.*;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.matchers.KeyMatcher;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Map;

@Component
public class ExcelSchedulerJobListener {

    private static Map<String, Scheduler> schedulerMap = Maps.newConcurrentMap();

    @Resource
    private TaskConfigService taskConfigService;

    public void execute(String jobName, String triggerName, TaskConfig taskConfig) throws Exception {
        TaskConfig config = taskConfigService.queryById(taskConfig.getId());
        if (config.getTaskStatus() == 0) {
            if (null != schedulerMap.get(jobName)) {
                schedulerMap.get(jobName).shutdown();
                schedulerMap.remove(jobName);
            } else {
                return;
            }
        } else {
            if (null == schedulerMap.get(jobName)) {
                Scheduler scheduler = StdSchedulerFactory.getDefaultScheduler();

                JobDataMap jobDataMap = new JobDataMap();
                jobDataMap.put("taskConfig", taskConfig);


                JobDetail jobDetail = JobBuilder.newJob(ExcelJob.class)
                        .withIdentity(jobName, "group1")
                        .usingJobData(jobDataMap)
                        .build();

                // 3、触发器 cron表达式 每5分钟执行一次
                Trigger trigger = TriggerBuilder.newTrigger()
                        .withIdentity(triggerName, "group1")
                        .withSchedule(CronScheduleBuilder.cronSchedule("0 */5 * * * ?"))
                        .build();

                // 4、让调度器关联任务和触发器，保证按照触发器定义的调整执行任务
                scheduler.scheduleJob(jobDetail, trigger);

                // 创建并注册一个局部的Job Listener，表示指定的任务Job
                scheduler.getListenerManager().addJobListener((JobListener) new MyJobListener(), KeyMatcher.keyEquals(JobKey.jobKey(jobName, "group1")));

                // 5、启动
                scheduler.start();

                schedulerMap.put(jobName, scheduler);
            }
        }
    }

}
