package com.ann.exceldemo.web.task;

import com.ann.exceldemo.web.constant.TaskTypeEnum;
import com.ann.exceldemo.web.dto.TaskConfig;
import com.ann.exceldemo.web.schedule.ExcelSchedulerJobListener;
import com.ann.exceldemo.web.service.TaskConfigService;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * excel自动化报表任务
 */

@Component
public class ExcelDealTask implements ApplicationRunner, Runnable {
    @Resource
    private TaskConfigService taskConfigService;
    @Resource
    private ExcelSchedulerJobListener excelSchedulerJobListener;

    @Override
    public void run(ApplicationArguments args) throws Exception {
        this.run();
    }

    @Override
    public void run() {
        // 创建 ScheduledThreadPool 线程池
        ScheduledExecutorService threadPool = Executors.newScheduledThreadPool(10);
        System.out.println("scheduleWithFixedDelay 方法添加任务：" + LocalDateTime.now());
        threadPool.scheduleWithFixedDelay(new Runnable() {
                                              @Override
                                              public void run() {
                                                  List<TaskConfig> taskConfigs = taskConfigService.queryAll();
                                                  for (TaskConfig taskConfig : taskConfigs) {
                                                      String taskType = taskConfig.getTaskType();
                                                      try {
                                                          if (TaskTypeEnum.order.getName().equals(taskType)) {
                                                              excelSchedulerJobListener.execute("orderJob", "orderTrigger", taskConfig);
                                                          } else if (TaskTypeEnum.waybill.getName().equals(taskType)) {
                                                              excelSchedulerJobListener.execute("waybillJob", "waybillTrigger", taskConfig);
                                                          }
                                                      } catch (Exception e) {
                                                          System.out.println("异常信息:" + e.getMessage());
                                                      }
                                                  }
                                              }
                                          },
                3L, // 3s 后开始执行定时任务
                20L, // 定时任务执行完 2s 之后，再执行下一个定时任务
                TimeUnit.SECONDS); // 描述上面两个参数的时间单位
    }

}
